package com.enjoyingfoss.feeel.view

import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.app.Service
import android.os.Build
import android.support.annotation.RequiresApi
import android.support.v4.app.NotificationCompat
import android.support.v4.app.NotificationManagerCompat
import android.support.v4.content.ContextCompat
import com.enjoyingfoss.feeel.R
import com.enjoyingfoss.feeel.WorkoutContract
import com.enjoyingfoss.feeel.model.ExerciseMeta
import java.lang.ref.WeakReference

class WorkoutNotification(initService: Service, activityIntent: PendingIntent) : WorkoutContract.View { //todo bug seconds shown wrong

    private val service = WeakReference<Service>(initService)

    companion object {
        private const val channelID = "com.enjoyingfoss.feeel.view.WorkoutNotification"

        @RequiresApi(Build.VERSION_CODES.O)
        fun createNotificationChannel(notificationManager: NotificationManager, name: String, description: String) {
            val importance = NotificationManager.IMPORTANCE_DEFAULT
            val channel = NotificationChannel(channelID, name, importance)
            channel.description = description
            notificationManager.createNotificationChannel(channel)
        }
    }

    private val notificationID = 1
    private val notificationManagerCompat = NotificationManagerCompat.from(initService)
    private val notificationBuilder = NotificationCompat.Builder(initService, channelID)
            .setVisibility(NotificationCompat.VISIBILITY_PUBLIC) // lock screen controls, with sensitive content
            .setSmallIcon(R.drawable.ic_notification)
            .setColor(ContextCompat.getColor(initService, R.color.colorPrimary))
            .setContentIntent(activityIntent)
            .setUsesChronometer(true)
            .setShowWhen(false)
    private val exerciseTitleTemplate = initService.getString(R.string.notification_title)
    private val breakTitleTemplate = initService.getString(R.string.next_label)

    private var secondsSet = false
    private var curSecondLimit = 0

    init {
        initService.startForeground(notificationID, notificationBuilder.build())
    }

    override fun setExercise(exerciseMeta: ExerciseMeta) {
        val exercise = exerciseMeta.exercise
        curSecondLimit = exerciseMeta.duration
        val notification = notificationBuilder
                .setWhen(System.currentTimeMillis() - 1000)
                .setContentTitle(String.format(exerciseTitleTemplate,
                        service.get()?.getString(exercise.titleResource),
                        curSecondLimit))
                .build()
        notificationManagerCompat.notify(notificationID, notification)
    }

    override fun setBreak(nextExerciseMeta: ExerciseMeta, breakLength: Int) {
        curSecondLimit = breakLength

        val service = service.get()
        if (service != null) {
            val nextExercise = nextExerciseMeta.exercise
            val notification = notificationBuilder
                    .setWhen(System.currentTimeMillis() - 1000)
                    .setContentTitle(
                            String.format(
                                    breakTitleTemplate,
                                    service.getString(nextExercise.titleResource)))
                    .build()
            notificationManagerCompat.notify(notificationID, notification)
        }
    }

    override fun showFinish() { //todo all unimplemented views
    }

    override fun setSeconds(seconds: Int) {
        if (!secondsSet) {
            val notification = notificationBuilder
                    .setWhen(System.currentTimeMillis() - (curSecondLimit - seconds) * 1000)
                    .build()
            notificationManagerCompat.notify(notificationID, notification)
            secondsSet = true
        }
    }

    override fun setPaused() {
        secondsSet = false
        val notification = notificationBuilder
                .setShowWhen(false)
                .build()
        notificationManagerCompat.notify(notificationID, notification)
    }

    override fun setPlaying() {
        val notification = notificationBuilder
                .setShowWhen(true)
                .build()
        notificationManagerCompat.notify(notificationID, notification)
    }

    override fun close() {} //todo anything here?
}